Imports Inventor
Imports System.Runtime.InteropServices
Imports Microsoft.Win32

Namespace UI_Ribbon_Demo
    <ProgIdAttribute("UI_RibbonSample.StandardAddInServer"), _
    GuidAttribute("3fe095f0-8eff-4cd0-b6a3-ac06556c3123")> _
    Public Class StandardAddInServer
        Implements Inventor.ApplicationAddInServer

        ' Inventor application object.
        Private m_inventorApplication As Inventor.Application

        Private WithEvents m_button1Def As Inventor.ButtonDefinition
        Private WithEvents m_button2Def As Inventor.ButtonDefinition
        Private WithEvents m_button3Def As Inventor.ButtonDefinition
        Private WithEvents m_button4Def As Inventor.ButtonDefinition
        Private WithEvents m_button5Def As Inventor.ButtonDefinition
        Private WithEvents m_button6Def As Inventor.ButtonDefinition

        Private WithEvents m_userInterfaceEvents As Inventor.UserInterfaceEvents

        Private m_ClientID As String


#Region "ApplicationAddInServer Members"

        Public Sub Activate(ByVal addInSiteObject As Inventor.ApplicationAddInSite, ByVal firstTime As Boolean) Implements Inventor.ApplicationAddInServer.Activate
            ' Initialize AddIn members.
            m_inventorApplication = addInSiteObject.Application

            ' Get the ClassID for this add-in and save it in a member variable to use wherever a ClientID is needed.
            m_ClientID = AddInGuid(GetType(StandardAddInServer))

            ' Set the large icon size based on whether it is the classic or ribbon interface.
            Dim UIManager As Inventor.UserInterfaceManager = m_inventorApplication.UserInterfaceManager
            Dim largeIconSize As Integer
            If UIManager.InterfaceStyle = InterfaceStyleEnum.kRibbonInterface Then
                largeIconSize = 32
            Else
                largeIconSize = 24
            End If

            ' Create the control definitions.  Multiple sizes of images can be saved in a single
            ' .ico file.  This extracts images by size from the icon file.
            Dim controlDefs As ControlDefinitions = m_inventorApplication.CommandManager.ControlDefinitions
            Dim smallPicture As stdole.IPictureDisp = Microsoft.VisualBasic.Compatibility.VB6.IconToIPicture(New System.Drawing.Icon(My.Resources.One, 16, 16))
            Dim largePicture As stdole.IPictureDisp = Microsoft.VisualBasic.Compatibility.VB6.IconToIPicture(New System.Drawing.Icon(My.Resources.One, largeIconSize, largeIconSize))
            m_button1Def = controlDefs.AddButtonDefinition("One", "UIRibbonSampleOne", CommandTypesEnum.kNonShapeEditCmdType, m_ClientID, , , smallPicture, largePicture)

            smallPicture = Microsoft.VisualBasic.Compatibility.VB6.IconToIPicture(New System.Drawing.Icon(My.Resources.Two, 16, 16))
            largePicture = Microsoft.VisualBasic.Compatibility.VB6.IconToIPicture(New System.Drawing.Icon(My.Resources.Two, largeIconSize, largeIconSize))
            m_button2Def = controlDefs.AddButtonDefinition("Two", "UIRibbonSampleTwo", CommandTypesEnum.kNonShapeEditCmdType, m_ClientID, , , smallPicture, largePicture)

            smallPicture = Microsoft.VisualBasic.Compatibility.VB6.IconToIPicture(New System.Drawing.Icon(My.Resources.Three, 16, 16))
            largePicture = Microsoft.VisualBasic.Compatibility.VB6.IconToIPicture(New System.Drawing.Icon(My.Resources.Three, largeIconSize, largeIconSize))
            m_button3Def = controlDefs.AddButtonDefinition("Three", "UIRibbonSampleThree", CommandTypesEnum.kNonShapeEditCmdType, m_ClientID, , , smallPicture, largePicture)

            smallPicture = Microsoft.VisualBasic.Compatibility.VB6.IconToIPicture(New System.Drawing.Icon(My.Resources.Four, 16, 16))
            largePicture = Microsoft.VisualBasic.Compatibility.VB6.IconToIPicture(New System.Drawing.Icon(My.Resources.Four, largeIconSize, largeIconSize))
            m_button4Def = controlDefs.AddButtonDefinition("Four", "UIRibbonSampleFour", CommandTypesEnum.kNonShapeEditCmdType, m_ClientID, , , smallPicture, largePicture)

            smallPicture = Microsoft.VisualBasic.Compatibility.VB6.IconToIPicture(New System.Drawing.Icon(My.Resources.Five, 16, 16))
            largePicture = Microsoft.VisualBasic.Compatibility.VB6.IconToIPicture(New System.Drawing.Icon(My.Resources.Five, largeIconSize, largeIconSize))
            m_button5Def = controlDefs.AddButtonDefinition("Five", "UIRibbonSampleFive", CommandTypesEnum.kNonShapeEditCmdType, m_ClientID, , , smallPicture, largePicture)

            smallPicture = Microsoft.VisualBasic.Compatibility.VB6.IconToIPicture(New System.Drawing.Icon(My.Resources.Six, 16, 16))
            largePicture = Microsoft.VisualBasic.Compatibility.VB6.IconToIPicture(New System.Drawing.Icon(My.Resources.Six, largeIconSize, largeIconSize))
            m_button6Def = controlDefs.AddButtonDefinition("Six", "UIRibbonSampleSix", CommandTypesEnum.kNonShapeEditCmdType, m_ClientID, , , smallPicture, largePicture)

            ' Create a command category and add the buttons to the category.
            Dim cmdCategory As CommandCategory
            cmdCategory = m_inventorApplication.CommandManager.CommandCategories.Add("Ribbon UI Demo", "UIRibbonSampleCategory", m_ClientID)
            cmdCategory.Add(m_button1Def)
            cmdCategory.Add(m_button2Def)
            cmdCategory.Add(m_button3Def)
            cmdCategory.Add(m_button4Def)
            cmdCategory.Add(m_button5Def)
            cmdCategory.Add(m_button6Def)

            If firstTime Then
                If UIManager.InterfaceStyle = InterfaceStyleEnum.kRibbonInterface Then
                    CreateOrUpdateRibbonUserInterface()
                ElseIf UIManager.InterfaceStyle = InterfaceStyleEnum.kClassicInterface Then
                    CreateClassicInterface()
                End If
            End If

            ' Connect to the user interface events to be able to handle when the user resets the interface.
            m_userInterfaceEvents = UIManager.UserInterfaceEvents
        End Sub


        Private Sub CreateOrUpdateRibbonUserInterface()
            ' Add the command to the File controls.
            Dim fileControls As CommandControls = m_inventorApplication.UserInterfaceManager.FileBrowserControls
            fileControls.AddButton(m_button3Def, , , "AppiPropertiesWrapperCmd")

            ' Get the assembly ribbon.
            Dim assemblyRibbon As Inventor.Ribbon = m_inventorApplication.UserInterfaceManager.Ribbons.Item("Assembly")

            ' Get the "Assemble" tab.
            Dim assembleTab As Inventor.RibbonTab = assemblyRibbon.RibbonTabs.Item("id_TabAssemble")

            ' Create a new panel on the Assemble tab.
            Dim panel As Inventor.RibbonPanel = assembleTab.RibbonPanels.Add("Ribbon UI Demo", "RibbonUIDemoPanel", m_ClientID)

            ' Add the buttons to the tab, with the first and last ones being large and the other two small.
            panel.CommandControls.AddButton(m_button1Def, True)
            panel.CommandControls.AddButton(m_button2Def)
            panel.CommandControls.AddButton(m_button5Def)
            panel.CommandControls.AddButton(m_button4Def, True)

            ' Add a button to the Help menu within the "Additional Resources" pop-up. 
            Dim additionalResourcePopUp As CommandControl = m_inventorApplication.UserInterfaceManager.HelpControls.Item("Additional Resources")
            additionalResourcePopUp.ChildControls.AddButton(m_button6Def)
        End Sub


        Private Sub CreateClassicInterface()
            ' Get the assembly environment.
            Dim UIManager As Inventor.UserInterfaceManager = m_inventorApplication.UserInterfaceManager
            Dim asmEnvironment As Environment
            asmEnvironment = UIManager.Environments.Item("AMxAssemblyEnvironment")

            ' Add a button to the assembly panel bar.
            Dim asmPanel As Inventor.CommandBar = asmEnvironment.PanelBar.DefaultCommandBar
            asmPanel.Controls.AddButton(m_button1Def)

            ' Add a button to the main tool bar of the assembly environment.
            asmEnvironment.DefaultToolBar.Controls.AddButton(m_button2Def)

            ' Add a button to the File menu right after the Close button.
            Dim fileBar As CommandBar = asmEnvironment.DefaultMenuBar.Controls.Item("AssemblyFileMenu").CommandBar
            fileBar.Controls.AddButton(m_button3Def, fileBar.Controls.Item("AppFileCloseCmd").Index + 1)

            ' Add a button to the Help menu at the bottom.
            Dim helpBar As CommandBar = asmEnvironment.DefaultMenuBar.Controls.Item("AppHelpMenu").CommandBar
            Dim additionalResourceButtonBar As CommandBar = helpBar.Controls.Item("AppHelpAdditionalResourcesMenu").CommandBar
            additionalResourceButtonBar.Controls.AddButton(m_button4Def)

            Dim commandBar As CommandBar = Nothing

            ' Need to check first to see if the command bar already exists.  If the user
            ' does a reset the command bar isn't deleted.
            Try
                commandBar = UIManager.CommandBars.Item("UISampleRegular")
            Catch ex As Exception
            End Try

            If commandBar Is Nothing Then
                ' The command bar doesn't exist, so create it.
                commandBar = UIManager.CommandBars.Add("Inventor Classic User Interface Demo", "UISampleRegular", CommandBarTypeEnum.kRegularCommandBar, m_ClientID)
            End If

            ' Add two buttons to it.
            commandBar.Controls.AddButton(m_button5Def)
            commandBar.Controls.AddButton(m_button6Def)

            ' Make the command bar visible.
            commandBar.Visible = True

            ' Add the new command bar to the panel bar list.
            asmEnvironment.PanelBar.CommandBarList.Add(commandBar)
        End Sub


        Public Sub Deactivate() Implements Inventor.ApplicationAddInServer.Deactivate
            ' Release objects.
            Marshal.ReleaseComObject(m_button1Def)
            m_button1Def = Nothing

            Marshal.ReleaseComObject(m_button2Def)
            m_button2Def = Nothing

            Marshal.ReleaseComObject(m_button3Def)
            m_button3Def = Nothing

            Marshal.ReleaseComObject(m_button4Def)
            m_button4Def = Nothing

            Marshal.ReleaseComObject(m_button5Def)
            m_button5Def = Nothing

            Marshal.ReleaseComObject(m_button6Def)
            m_button6Def = Nothing

            Marshal.ReleaseComObject(m_userInterfaceEvents)
            m_userInterfaceEvents = Nothing

            Marshal.ReleaseComObject(m_inventorApplication)
            m_inventorApplication = Nothing

            System.GC.WaitForPendingFinalizers()
            System.GC.Collect()
        End Sub

        Public ReadOnly Property Automation() As Object Implements Inventor.ApplicationAddInServer.Automation
            Get
                Return Nothing
            End Get
        End Property

        Public Sub ExecuteCommand(ByVal commandID As Integer) Implements Inventor.ApplicationAddInServer.ExecuteCommand
        End Sub

#End Region

#Region "COM Registration"

        ' Registers this class as an AddIn for Inventor.
        ' This function is called when the assembly is registered for COM.
        <ComRegisterFunctionAttribute()> _
        Public Shared Sub Register(ByVal t As Type)

            Dim clssRoot As RegistryKey = Registry.ClassesRoot
            Dim clsid As RegistryKey = Nothing
            Dim subKey As RegistryKey = Nothing

            Try
                clsid = clssRoot.CreateSubKey("CLSID\" + AddInGuid(t))
                clsid.SetValue(Nothing, "Ribbon UI Demo")
                subKey = clsid.CreateSubKey("Implemented Categories\{39AD2B5C-7A29-11D6-8E0A-0010B541CAA8}")
                subKey.Close()

                subKey = clsid.CreateSubKey("Settings")
                subKey.SetValue("AddInType", "Standard")
                subKey.SetValue("LoadOnStartUp", "1")

                'subKey.SetValue("SupportedSoftwareVersionLessThan", "")
                subKey.SetValue("SupportedSoftwareVersionGreaterThan", "12..")
                'subKey.SetValue("SupportedSoftwareVersionEqualTo", "")
                'subKey.SetValue("SupportedSoftwareVersionNotEqualTo", "")
                'subKey.SetValue("Hidden", "0")
                'subKey.SetValue("UserUnloadable", "1")
                subKey.SetValue("Version", 1)
                subKey.Close()

                subKey = clsid.CreateSubKey("Description")
                subKey.SetValue(Nothing, "Demonstrates the use of the ribbon portion of the Inventor API.")

            Catch ex As Exception
                System.Diagnostics.Trace.Assert(False)
            Finally
                If Not subKey Is Nothing Then subKey.Close()
                If Not clsid Is Nothing Then clsid.Close()
                If Not clssRoot Is Nothing Then clssRoot.Close()
            End Try

        End Sub

        ' Unregisters this class as an AddIn for Inventor.
        ' This function is called when the assembly is unregistered.
        <ComUnregisterFunctionAttribute()> _
        Public Shared Sub Unregister(ByVal t As Type)

            Dim clssRoot As RegistryKey = Registry.ClassesRoot
            Dim clsid As RegistryKey = Nothing

            Try
                clssRoot = Microsoft.Win32.Registry.ClassesRoot
                clsid = clssRoot.OpenSubKey("CLSID\" + AddInGuid(t), True)
                clsid.SetValue(Nothing, "")
                clsid.DeleteSubKeyTree("Implemented Categories\{39AD2B5C-7A29-11D6-8E0A-0010B541CAA8}")
                clsid.DeleteSubKeyTree("Settings")
                clsid.DeleteSubKeyTree("Description")
            Catch
            Finally
                If Not clsid Is Nothing Then clsid.Close()
                If Not clssRoot Is Nothing Then clssRoot.Close()
            End Try

        End Sub

        ' This property uses reflection to get the value for the GuidAttribute attached to the class.
        Public Shared ReadOnly Property AddInGuid(ByVal t As Type) As String
            Get
                Dim guid As String = ""
                Try
                    Dim customAttributes() As Object = t.GetCustomAttributes(GetType(GuidAttribute), False)
                    Dim guidAttribute As GuidAttribute = CType(customAttributes(0), GuidAttribute)
                    guid = "{" + guidAttribute.Value.ToString() + "}"
                Finally
                    AddInGuid = guid
                End Try
            End Get
        End Property

#End Region

        Private Sub m_button1Def_OnExecute(ByVal Context As Inventor.NameValueMap) Handles m_button1Def.OnExecute
            MsgBox("Button 1 executed")
        End Sub

        Private Sub m_button2Def_OnExecute(ByVal Context As Inventor.NameValueMap) Handles m_button2Def.OnExecute
            MsgBox("Button 2 executed")
        End Sub

        Private Sub m_button3Def_OnExecute(ByVal Context As Inventor.NameValueMap) Handles m_button3Def.OnExecute
            MsgBox("Button 3 executed")
        End Sub

        Private Sub m_button4Def_OnExecute(ByVal Context As Inventor.NameValueMap) Handles m_button4Def.OnExecute
            MsgBox("Button 4 executed")
        End Sub

        Private Sub m_button5Def_OnExecute(ByVal Context As Inventor.NameValueMap) Handles m_button5Def.OnExecute
            MsgBox("Button 5 executed")
        End Sub

        Private Sub m_button6Def_OnExecute(ByVal Context As Inventor.NameValueMap) Handles m_button6Def.OnExecute
            MsgBox("Button 6 executed")
        End Sub

        Private Sub m_userInterfaceEvents_OnResetCommandBars(ByVal CommandBars As Inventor.ObjectsEnumerator, ByVal Context As Inventor.NameValueMap) Handles m_userInterfaceEvents.OnResetCommandBars
            ' Get the assembly environment.
            Dim UIManager As Inventor.UserInterfaceManager = m_inventorApplication.UserInterfaceManager
            Dim asmEnvironment As Environment
            asmEnvironment = UIManager.Environments.Item("AMxAssemblyEnvironment")

            For Each commandBar As Inventor.CommandBar In CommandBars
                ' Special case for each command bar.
                If commandBar Is asmEnvironment.PanelBar.DefaultCommandBar Then
                    commandBar.Controls.AddButton(m_button1Def)
                End If

                If commandBar Is asmEnvironment.DefaultToolBar Then
                    commandBar.Controls.AddButton(m_button2Def)
                End If

                If commandBar Is asmEnvironment.DefaultMenuBar.Controls.Item("AssemblyFileMenu").CommandBar Then
                    ' Add a button to the File menu right after the Close button.
                    commandBar.Controls.AddButton(m_button3Def, commandBar.Controls.Item("AppFileCloseCmd").Index + 1)
                End If

                Dim helpBar As CommandBar = asmEnvironment.DefaultMenuBar.Controls.Item("AppHelpMenu").CommandBar
                If commandBar Is helpBar.Controls.Item("AppHelpAdditionalResourcesMenu").CommandBar Then
                    ' Add a button to the Help menu at the bottom.
                    Dim additionalResourceButtonBar As CommandBar = helpBar.Controls.Item("AppHelpAdditionalResourcesMenu").CommandBar
                    additionalResourceButtonBar.Controls.AddButton(m_button4Def)
                End If

                If commandBar.InternalName = "UISampleRegular" Then
                    ' Add two buttons to it.
                    commandBar.Controls.AddButton(m_button5Def)
                    commandBar.Controls.AddButton(m_button6Def)

                    ' Make the command bar visible.
                    commandBar.Visible = True
                End If
            Next
        End Sub

        Private Sub m_userInterfaceEvents_OnResetEnvironments(ByVal Environments As Inventor.ObjectsEnumerator, ByVal Context As Inventor.NameValueMap) Handles m_userInterfaceEvents.OnResetEnvironments
            Dim UIManager As Inventor.UserInterfaceManager = m_inventorApplication.UserInterfaceManager
            Dim asmEnvironment As Environment = UIManager.Environments.Item("AMxAssemblyEnvironment")

            Dim commandBar As CommandBar
            Try
                commandBar = UIManager.CommandBars.Item("UISampleRegular")
            Catch ex As Exception
                commandBar = UIManager.CommandBars.Add("Inventor Classic User Interface Demo", "UISampleRegular", CommandBarTypeEnum.kRegularCommandBar, m_ClientID)
            End Try

            ' Add the new command bar to the panel bar list.
            asmEnvironment.PanelBar.CommandBarList.Add(commandBar)
        End Sub

        Private Sub m_userInterfaceEvents_OnResetRibbonInterface(ByVal Context As Inventor.NameValueMap) Handles m_userInterfaceEvents.OnResetRibbonInterface
            CreateOrUpdateRibbonUserInterface()
        End Sub
    End Class

End Namespace

